﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- These functions provide integration with and management of Recount which
-- provides for damage and healing meters, etc.
-------------------------------------------------------------------------------

-- local cache

local CreateFrame      = CreateFrame;
local InCombatLockdown = InCombatLockdown;

-------------------------------------------------------------------------------

function nUI:initRecount()

	if nUI.AddOn.recount then
		
		local frame = Recount.MainWindow;
			
		if not frame.nUI_init then
			
			frame.nUI_init = true;
				
			nUI:addInfoMode( "Stats", 
				function() 
					frame:Show(); 
				end, 
				function() 
					local mode = nUI.info_frame.modeName[nUI_Config.activeInfoMode];
					
					if mode ~= "Stats" and mode ~= "KTM" and mode ~= "Omen2" then
						frame:Hide(); 
					elseif InCombatLockdown() then
						frame:Hide();
					else
						frame:Show();
					end 
				end
			);
			
			frame:SetParent( nUI.info_frame.stats );
			frame:SetPoint( "TOPLEFT", nUI.info_frame.stats, "TOPLEFT", 0, 11 );
			frame:SetPoint( "BOTTOMRIGHT", nUI.info_frame.stats, "BOTTOMRIGHT", -5, 0 );
			frame:SetWidth( frame:GetRight() - frame:GetLeft() ); 
			frame:SetFrameStrata( nUI.info_frame.stats:GetFrameStrata() );
			frame:SetFrameLevel( nUI.info_frame.stats:GetFrameLevel()+1 );
			frame:SetHeight( frame:GetTop() - frame:GetBottom() ); 
			frame:SetBackdropBorderColor( 0, 0, 0, 1 );
			frame:SetBackdropColor( 0, 0, 0, 0 );
			frame:Show();
			nUI:lockFrame( frame, true, nil );
			
			frame.CloseButton:Hide();
			
			Recount:ResizeMainWindow();
			Recount:LockWindows( true );
			
			local event_frame = CreateFrame( "Frame", "$parent_RecountEvents", nUI.info_frame );
			nUI.info_frame.recount_events = event_frame;
			
			event_frame:SetScript( "OnEvent", function() nUI:recountEvent(); end );
			
			event_frame:RegisterEvent( "PLAYER_REGEN_ENABLED" );
			event_frame:RegisterEvent( "PLAYER_REGEN_DISABLED" );
			event_frame:RegisterEvent( "PET_ATTACK_START" );
			event_frame:RegisterEvent( "PET_ATTACK_STOP" );
		end
	end
end

-------------------------------------------------------------------------------

function nUI:recountEvent()
	
	local mode  = nUI.info_frame.modeName[nUI_Config.activeInfoMode];
	local frame = Recount.MainWindow;
	
	if event == "PLAYER_REGEN_DISABLED" or event == "PET_ATTACK_START" then
		if frame:IsShown() and mode ~= "Stats" then
			frame:Hide();
		end
	elseif event == "PLAYER_REGEN_ENABLED" or (event == "PET_ATTACK_STOP" and not InCombatLockdown()) then
		if not frame:IsShown() and (mode == "Stats" or mode == "Omen2" or mode == "KTM") then
			frame:Show();
		end
	end
end
