﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------
	
-------------------------------------------------------------------------------
-- local cache

local CreateFrame             = CreateFrame;
local GetRestState            = GetRestState;
local GetXPExhaustion         = GetXPExhaustion;
local UnitXP                  = UnitXP;
local UnitXPMax               = UnitXPMax;
local UnitLevel               = UnitLevel;
local UIFrameFlash            = UIFrameFlash;
local IsResting               = IsResting;
local UIFrameFlashRemoveFrame = UIFrameFlashRemoveFrame;

-- note the last value in this table is a dummy value

nUI_XpPerLevel = 
{
	   400,    900,   1400,   2100,   2800,   3600,   4400,   5400,   6500,    7600,
	  8700,   9800,  11000,  12300,  13600,  15000,  16400,  17800,  19300,   20800,
	 22400,  24000,  25500,  27200,  28900,  30500,  32200,  33900,  36300,   38800, 
	 41600,  44600,  48000,  51400,  55000,  58700,  62400,  66200,  70200,   74300,
	 78500,  82800,  87100,  91600,  95300, 101000, 105800, 110700, 115700,  120900,
	126100, 131500, 137000, 142500, 148200, 154000, 159900, 165800, 172000,  494000,
	574700, 614400, 650300, 682300, 710200, 734100, 753700, 768900, 779700,  810888 
};

-------------------------------------------------------------------------------

nUI.Class.XPBar = nUI.Class:CreatePrototype( nUI.Class.StatusBar );
local prototype = nUI.Class.XPBar.prototype;

-------------------------------------------------------------------------------

function prototype:layoutFrame( layout, is_init )
	
	if not is_init then
		self:unregisterEvents();
		prototype.super.prototype.layoutFrame( self, layout );
	else
		self.event_frame  = CreateFrame( "Frame", layout.name.."_EventFrame", self.parent );

		prototype.super.prototype.layoutFrame( self, layout, true );
		
		self.frame.grid   = CreateFrame( "Frame", "$parent_Grid", self.frame );
		self.frame.rested = CreateFrame( "Frame", self.layout.name.."_Rested", self.frame );
		self.frame.marker = CreateFrame( "Frame", self.layout.name.."_XPRested", self.frame );

		self.frame.grid.texture   = self.frame.grid:CreateTexture();
		self.frame.rested.texture = self.frame.rested:CreateTexture();
		self.frame.marker.texture = self.frame.marker:CreateTexture();

		self.frame.grid.texture:SetTexture( "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_XPRepOverlay.blp" );
		self.frame.grid.texture:SetTexCoord( 0, 0, 0, 1, 0.978515625, 0, 0.978515625, 1 );
		
		self.frame.marker.texture:SetTexture( "Interface\\AddOns\\nUI\\Bars\\Art\\nUI_XPRested.blp" );
		self.frame.marker.texture:SetTexCoord( 0, 0, 0, 1, 1, 0, 1, 1 );
	end
		
	self.frame:SetPoint( self.layout.Origin.anchor, nUI_Dashboard, "CENTER", self.layout.Origin.xOfs, self.layout.Origin.yOfs );
	
	self.frame:SetBackdrop(
		{
			bgFile   = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBg.blp", 
			edgeFile = "Interface\\AddOns\\nUI\\Art\\nUI_BevelboxBorder.blp", 
			tile     = true, 
			tileSize = 1, 
			edgeSize = 12, 
			insets   = {left = 0, right = 0, top = 0, bottom = 0},
		}
	);

	self.frame:SetBackdropColor( 0, 0, 0, 0.35 );
	
	self.frame.grid:SetHeight( self.frame:GetHeight() - 2 );
	self.frame.grid:SetWidth( self.frame:GetWidth() - 2 );
	self.frame.grid:SetPoint( "CENTER", self.frame, "CENTER", 0, 0 );
	self.frame.grid:SetFrameLevel( self.frame.bar:GetFrameLevel()+1 );
	self.frame.grid.texture:SetAllPoints( self.frame.grid );

	self.frame.bar.center:SetScale( 1.0 / nUI.scale );
	self.frame.bar.center.text:ClearAllPoints();
	self.frame.bar.center.text:SetFont( "Fonts\\ARIALN.TTF", self.layout.Label.size * nUI.scale, "OUTLINE" );
	self.frame.bar.center.text:SetPoint( self.layout.Label.anchor, self.frame, "CENTER", self.layout.Label.xOfs * nUI.scale, self.layout.Label.yOfs * nUI.scale );
	
	self.frame.rested:SetHeight( self.frame.grid:GetHeight() );
	self.frame.rested:SetWidth( self.frame.grid:GetWidth() );
	self.frame.rested:SetFrameLevel( self.frame.bar:GetFrameLevel()-1 );
	self.frame.rested:SetAllPoints( self.frame.grid );
	
	self.frame.rested.texture:SetHeight( self.frame.rested:GetHeight() );
	self.frame.rested.texture:SetTexture( self.layout.Colors.Rested.r, self.layout.Colors.Rested.g, self.layout.Colors.Rested.b, 1 );
	
	self.frame.marker:SetHeight( self.frame:GetHeight() * 2 );
	self.frame.marker:SetWidth( self.frame:GetHeight() * 2 );
	self.frame.marker:SetPoint( "CENTER", self.frame.bar, "LEFT", 0, 0 );
	self.frame.marker:SetFrameLevel( self.frame.grid:GetFrameLevel() + 1 );
	self.frame.marker:Hide();
	
	self.frame.marker.texture:SetAllPoints( self.frame.marker );

	self:registerEvents();
	self:updateXP();
	
end

-------------------------------------------------------------------------------

function prototype:registerEvents()

	prototype.super.prototype.registerEvents( self );
	
	self.event_frame:SetScript( "OnEvent", function() self:onEvent(); end );
	self.event_frame:RegisterEvent( "PLAYER_XP_UPDATE" );
	self.event_frame:RegisterEvent( "UPDATE_EXHAUSTION" );
	self.event_frame:RegisterEvent( "PLAYER_LEVEL_UP" );
	self.event_frame:RegisterEvent( "PLAYER_UPDATE_RESTING" );

end

-------------------------------------------------------------------------------

function prototype:unregisterEvents( is_init )

	prototype.super.prototype.unregisterEvents( self );
	
	self.event_frame:SetScript( "OnEvent", nil );
	self.event_frame:UnregisterEvent( "PLAYER_XP_UPDATE" );
	self.event_frame:UnregisterEvent( "UPDATE_EXHAUSTION" );
	self.event_frame:UnregisterEvent( "PLAYER_LEVEL_UP" );
	self.event_frame:UnregisterEvent( "PLAYER_UPDATE_RESTING" );

end

-------------------------------------------------------------------------------

function prototype:onEvent()
	
	if     event == "PLAYER_XP_UPDATE"      then self:updateXP();
	elseif event == "UPDATE_EXHAUSTION"     then self:updateXP();
	elseif event == "PLAYER_LEVEL_UP"       then self:updateXP( arg1 );
	elseif event == "PLAYER_UPDATE_RESTING" then self:updateXP();
	end
end

-------------------------------------------------------------------------------

function prototype:updateXP( level )
	
	local player_lvl = level or UnitLevel( "player" );
	
	if player_lvl == MAX_PLAYER_LEVEL then
		
		if self.frame:IsShown() then self:Hide(); end
		
	else

		local is_rested, name, mult  = GetRestState();
		local resting    = IsResting();
		local rested_xp  = GetXPExhaustion();
		local xp         = UnitXP( "player" );
		local xp_max     = UnitXPMax( "player" );
		local label      = self.frame.bar.center;
		local txt_color;
		local bar_color  = self.layout.Colors.Normal;
		
		if not self.frame:IsShown() then self:Show(); end

		if rested_xp and rested_xp > 0 then
			txt_color = self.layout.Label.Colors.Rested;
		else
			txt_color = self.layout.Label.Colors.Normal;
		end
		
		self:updateValues( xp, 0, 0, xp_max );
		self.frame.bar.texture:SetTexture( bar_color.r, bar_color.g, bar_color.b, 1 );		
		
		label.text:SetText( ("Level "..player_lvl..": "..xp.." of "..xp_max.." (%0.1f%%), "..(rested_xp or 0).." rested XP"):format(xp / xp_max * 100) );
		label.text:SetTextColor( txt_color.r, txt_color.g, txt_color.b, 1 );

		-- if we have some rested XP, then place a marker in the appropriate place
		
		if rested_xp and rested_xp > 0 then
			
			if not self.frame.marker:IsShown() then 
				self.frame.rested:Show();
				self.frame.marker:Show();
				self.frame.marker:SetAlpha( 1 );
				UIFrameFlash( self.frame.marker, 0.5, 0.5, 86400, true, 0, 0 );
			end
			
			local offset = xp + rested_xp;
			local level  = player_lvl;
			local dX     = offset / xp_max;

			while offset > nUI_XpPerLevel[level] do 
				offset = offset - nUI_XpPerLevel[level]; 
				dX     = offset / nUI_XpPerLevel[level+1];
				level  = level+1; 
			end
			
			-- we only display the rested XP end marker if the marker is on the 
			-- current level as the player or the player is less than lvl 69. Otherwise,
			-- there is no such thing as wraparound rested XP for a lvl69 player.
			
			if not (level > player_lvl and player_lvl+1 == MAX_PLAYER_LEVEL) then
					
				self.frame.marker:ClearAllPoints();
				
				if self.layout.Options.grow ~= "left" then				
					self.frame.marker:SetPoint( "CENTER", self.frame.bar, "LEFT", dX * self.frame:GetWidth(), 0 );
				else
					self.frame.marker:SetPoint( "CENTER", self.frame.bar, "RIGHT", -dX * self.frame:GetWidth(), 0 );
				end
				
			else
				
				self.frame.marker:Hide();
				
			end
			
			-- draw a rested XP bar behind the XP bar
			
			if xp + rested_xp > xp_max then
				
				-- just fill the whole bar
		
				self.frame.rested.texture:ClearAllPoints();
				self.frame.rested:SetWidth( self.frame.grid:GetWidth() );
				self.frame.rested.texture:SetAllPoints( self.frame.grid );
				
			else
				
				-- draw a partial bar from the end of the current XP to 
				-- XP + rested XP
				
				self.frame.rested.texture:ClearAllPoints();
				self.frame.rested.texture:SetWidth( (xp+rested_xp) / xp_max * self.frame.grid:GetWidth() );
				
				if self.layout.Options.grow ~= "left" then
					self.frame.rested.texture:SetPoint( "LEFT", self.frame.grid, "LEFT", 0, 0 );
				else
					self.frame.rested.texture:SetPoint( "RIGHT", self.frame.grid, "RIGHT", 0, 0 );
				end
			end
			
		elseif self.frame.marker:IsShown() then
			
			UIFrameFlashRemoveFrame( self.frame.marker );
			self.frame.marker:Hide();
			self.frame.rested:Hide();
			
		end
	end
end
