﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

nUI_Movers = {};

-------------------------------------------------------------------------------

function nUI:initMovers()

	-- standard Bliz frames we want to  move
	
	QuestWatchFrame:ClearAllPoints();
	QuestWatchFrame:SetPoint( "RIGHT", UIParent, "RIGHT", -5, 256 * nUI.scale );	
	nUI:lockFrame( QuestWatchFrame, true, "Watched Quest" );

	QuestTimerFrame:ClearAllPoints();
	QuestTimerFrame:SetPoint( "TOP", nUI_Console, "BOTTOM", 0, -5 );
	nUI:lockFrame( QuestTimerFrame, true, "Quest Timer" );
	
	DurabilityFrame:ClearAllPoints();
	DurabilityFrame:SetPoint( "TOP", QuestTimerFrame, "BOTTOM", 0, -5 );
	nUI:lockFrame( DurabilityFrame, true, "Durability" );

	-- create a mover for the capture bars
	
	local frame = WorldStateCaptureBar1;
	local mover = nUI_Movers["nUI_CaptureBarMover"];
	
	nUI_CaptureBarMover = CreateFrame( "Frame", "nUI_CaptureBarMover", UIParent );
	nUI_CaptureBarMover:SetWidth( 173 );
	nUI_CaptureBarMover:SetHeight( 26 );
	
	if not mover then
		nUI_CaptureBarMover:SetPoint( "TOPRIGHT", UIParent, "TOPRIGHT", -10, -20 );
	else
		for i in pairs( mover.Point ) do
			local point = mover.Point[i];
			nUI_CaptureBarMover:SetPoint( point.anchor, point.relative_to, point.relative_pt, point.xOfs, point.yOfs );
		end
	end
	
	nUI_CaptureBarMover:SetScript( "OnEvent", 
		function()	
			if event == "UPDATE_WORLD_STATES" then
				for i=1,NUM_EXTENDED_UI_FRAMES do
					local frame = _G["WorldStateCaptureBar"..i];				
					if not frame.fCache then nUI:lockFrame( frame, true, nil ); end					
					frame.fCache.overlay:SetPoint( "CENTER", nUI_CaptureBarMover, "CENTER", 0, 0 );
				end
			end
		end
	);

	nUI:lockFrame( nUI_CaptureBarMover, true, "Capture Bar" );
	nUI_CaptureBarMover:RegisterEvent( "UPDATE_WORLD_STATES" );	
	
end

-------------------------------------------------------------------------------

function nUI:lockFrame( frame, mode, overlay )

	local new_mover = false;
	
	-- if we don't already have a frame cache for this frame, initialize it
	
	if not frame.fCache then	

		new_mover                   = true;		
		frame.fCache                = {};		
		frame.fCache_SetAllPoints   = frame.SetAllPoints;
		frame.fCache_SetPoint       = frame.SetPoint;
		frame.fCache_ClearAllPoints = frame.ClearAllPoints;

	end
	
	-- lock the frame to prevent anyone else from moving it
	
	if not frame.fCache.locked and mode then
		
		frame.fCache.locked  = true;
		frame.SetAllPoints   = function() end;
		frame.SetPoint       = function() end;
		frame.ClearAllPoints = function() end;
	
	-- make the frame moveable again
	
	elseif frame.fCache.locked and not mode then

		frame.fCache.locked  = false;
		frame.SetAllPoints   = frame.fCache_SetAllPoints;
		frame.SetPoint       = frame.fCache_SetPoint;
		frame.ClearAllPoints = frame.fCache_ClearAllPoints;
		
	end

	-- if this is a new mover, set up the overlay for it
	
	if new_mover then
		
		local frame_name = frame:GetName();
		local mover      = nUI_Movers[frame_name];		

		-- if thise frame has a visible overlay, then add it to the list of 
		-- movers we enable/disable on the "/nui movers" slash command

		if overlay then
			if not nUI.Movers then nUI.Movers = {}; end
			if not nUI.Movers[frame_name] then nUI.Movers[frame_name] = frame; end
		end
		
		-- create a default mover in the config if we don't have one yet
		
		if not mover then
			
			mover = {};
			mover.Point = {};
			
			for i=1,frame:GetNumPoints() do
				
				local point = {};

				point.anchor, point.relative_to, point.relative_pt, point.xOfs, point.yOfs = frame:GetPoint( i );
				point.relative_to = point.relative_to and point.relative_to:GetName() or point_relative_to;
				
				mover.Point[i] = point;
				
			end
			
			nUI_Movers[frame_name] = mover;
			
		end

		frame.fCache.label = overlay;	
		frame.fCache.drag  = true;
		
		-- create an overlay mover frame to show the user and/or for dragging the frame
	
		frame.fCache.overlay = CreateFrame( "Frame", nil, frame:GetParent() );
		
		frame.fCache.overlay:SetFrameStrata( frame:GetFrameStrata() );
		frame.fCache.overlay:SetFrameLevel( frame:GetFrameLevel()+1 );
		
		frame.fCache.overlay:SetWidth( frame:GetWidth() );
		frame.fCache.overlay:SetHeight( frame:GetHeight() );

		for i in pairs( mover.Point ) do
			local point = mover.Point[i];
			frame.fCache.overlay:SetPoint( point.anchor, point.relative_to, point.relative_pt, point.xOfs, point.yOfs );
		end
			
		frame.fCache.overlay:EnableMouse( true );
		frame.fCache.overlay:RegisterForDrag();
		frame.fCache.overlay:SetScript( "OnDragStart", function() nUI:dragFrame( frame, true ); end );
		frame.fCache.overlay:SetScript( "OnDragStop", function() nUI:dragFrame( frame, false ); end );
		frame.fCache.overlay:SetMovable( true );

		-- if we want the mover to be visible, creat a texture for it
		
		if overlay then
			
			frame.fCache.overlay.texture = frame.fCache.overlay:CreateTexture();
			frame.fCache.overlay.texture:SetAllPoints( frame.fCache.overlay );
			frame.fCache.overlay.texture:SetTexture( 0.5, 0.5, 0.5, 0.5 );
			
		end

		-- lock the frame to the overlay
		
		frame:fCache_ClearAllPoints();
		
		for i in pairs( mover.Point ) do
			local point = mover.Point[i];
			frame:fCache_SetPoint( point.anchor, frame.fCache.overlay, point.anchor, 0, 0 );
		end
		
		-- disable any other movers that may be attached to the frame
		
		frame:RegisterForDrag();
		frame:SetScript( "OnDragStart", nil );
		frame:SetScript( "OnDragStop", nil );

		-- disable dragging
		
		self:enableDrag( frame, false );
	end	
end

-------------------------------------------------------------------------------

function nUI:enableDrag( frame, mode )

	if mode then

		if not frame.fCache.moving and not frame.fCache.drag then
			
			frame.fCache.drag = true;
			frame.fCache.overlay:SetWidth( max( frame:GetWidth(), 25 ) );
			frame.fCache.overlay:SetHeight( max( frame:GetHeight(), 25 ) );
			frame.fCache.overlay:SetScale( frame:GetScale() );
			frame.fCache.overlay:Show();
			frame.fCache.overlay:RegisterForDrag( "LeftButton" );
			frame.fCache.overlay:SetScript( "OnEnter", function() GameTooltip:SetText( "Left click to drag "..frame.fCache.label ); GameTooltip:SetOwner( frame.fCache.overlay, "ANCHOR_BOTTOMRIGHT", 0, 0 ); GameTooltip:Show() end );
			frame.fCache.overlay:SetScript( "OnLeave", function() GameTooltip:Hide(); end );
			
		end
			
	else
		
		if not frame.fCache.moving and frame.fCache.drag then
			
			frame.fCache.drag = false;
			frame.fCache.overlay:Hide();
			frame.fCache.overlay:RegisterForDrag();
			frame.fCache.overlay:SetScript( "OnEnter", function() end );
			frame.fCache.overlay:SetScript( "OnLeave", function() end );
		end		
	end
end

-------------------------------------------------------------------------------

function nUI:dragFrame( frame, mode )
	
	if mode and frame.fCache.drag and not frame.fCache.moving then
		
		frame.fCache.overlay:StartMoving();
		frame.fCache.moving = true;
		GameTooltip:Hide();
		
	elseif not mode and frame.fCache.drag and frame.fCache.moving then
		
		local mover   = nUI_Movers[frame:GetName()];
		local overlay = frame.fCache.overlay;
		
		overlay:StopMovingOrSizing();
		frame.fCache.moving = false;

		mover.Point = {};
		
		for i=1,overlay:GetNumPoints() do
			
			local point = {};

			point.anchor, point.relative_to, point.relative_pt, point.xOfs, point.yOfs = overlay:GetPoint( i );
			point.relative_to = point.relative_to and point.relative_to:GetName() or point_relative_to;
			
			mover.Point[i] = point;
			
		end
	end
end
 
-------------------------------------------------------------------------------

function nUI:toggleMovers()
	
	if nUI.Movers then
			
		if not nUI.movers_enabled then nUI.movers_enabled = true;
		else nUI.movers_enabled = false;
		end
	
		DEFAULT_CHAT_FRAME:AddMessage( "nUI: Bliz frame movers "..(nUI.movers_enabled and "|cFF00FF00ENABLED|r" or "|cFFFF0000DISABLED|r"), 0.75, 0.75, 0.75 );
		
		for i in pairs( nUI.Movers ) do
			
			nUI:enableDrag( nUI.Movers[i], nUI.movers_enabled );
			
		end
	end
end